@@ -27,7 +27,6 @@ and WildFire appliance.
 # decided to not require non-default modules.  That decision may need
 # to be revisited as some parts of this are not clean.
 
-from __future__ import print_function
 import socket
 import sys
 import os
@@ -36,21 +35,12 @@ import email
 import email.errors
 import email.utils
 import logging
-try:
-    # 3.2
-    from urllib.request import Request, \
-        build_opener, HTTPErrorProcessor, HTTPSHandler
-    from urllib.error import URLError
-    from urllib.parse import urlencode
-    from http.client import responses
-    _legacy_urllib = False
-except ImportError:
-    # 2.7
-    from urllib2 import Request, URLError, \
-        build_opener, HTTPErrorProcessor, HTTPSHandler
-    from urllib import urlencode
-    from httplib import responses
-    _legacy_urllib = True
+
+from urllib.request import Request, \
+    build_opener, HTTPErrorProcessor, HTTPSHandler
+from urllib.error import URLError
+from urllib.parse import urlencode
+from http.client import responses
 
 import xml.etree.ElementTree as etree
 from . import __version__, DEBUG1, DEBUG2, DEBUG3
@@ -78,6 +68,7 @@ BENIGN = 0
 MALWARE = 1
 GRAYWARE = 2
 PHISHING = 4
+C2 = 5
 PENDING = -100
 ERROR = -101
 UNKNOWN = -102
@@ -88,6 +79,7 @@ VERDICTS = {
     MALWARE: ('malware', None),
     GRAYWARE: ('grayware', None),
     PHISHING: ('phishing', None),
+    C2: ('C2', 'command-and-control'),
     PENDING: ('pending', 'sample exists and verdict not known'),
     ERROR: ('error', 'sample is in error state'),
     UNKNOWN: ('unknown', 'sample does not exist'),
@@ -95,28 +87,6 @@ VERDICTS = {
 }
 
 
-def _isunicode(s):
-    try:
-        if isinstance(s, unicode):
-            return True
-        return False
-    except NameError:
-        if isinstance(s, str):
-            return True
-        return False
-
-
-def _isbytes(s):
-    try:
-        if isinstance(s, basestring) and isinstance(s, bytes):
-            return True
-        return False
-    except NameError:
-        if isinstance(s, bytes):
-            return True
-        return False
-
-
 class PanWFapiError(Exception):
     pass
 
@@ -128,13 +98,15 @@ class PanWFapi:
                  api_key=None,
                  timeout=None,
                  http=False,
-                 ssl_context=None):
+                 ssl_context=None,
+                 agent=None):
         self._log = logging.getLogger(__name__).log
         self.tag = tag
         self.hostname = hostname
         self.api_key = None
         self.timeout = timeout
         self.ssl_context = ssl_context
+        self.agent = agent
 
         self._log(DEBUG3, 'Python version: %s', sys.version)
         self._log(DEBUG3, 'xml.etree.ElementTree version: %s', etree.VERSION)
@@ -149,25 +121,16 @@ class PanWFapi:
             except ValueError:
                 raise PanWFapiError('Invalid timeout: %s' % self.timeout)
 
-        if self.ssl_context is not None:
-            try:
-                ssl.SSLContext(ssl.PROTOCOL_SSLv23)
-            except AttributeError:
-                raise PanWFapiError('SSL module has no SSLContext()')
-        elif _have_certifi:
+        if self.ssl_context is None and _have_certifi:
             self.ssl_context = self._certifi_ssl_context()
 
-        # handle Python versions with no ssl.CertificateError
-        if hasattr(ssl, 'CertificateError'):
-            self._certificateerror = ssl.CertificateError
-        else:
-            self._certificateerror = NotImplementedError  # XXX Can't happen
-
         init_panrc = {}  # .panrc args from constructor
         if hostname is not None:
             init_panrc['hostname'] = hostname
         if api_key is not None:
             init_panrc['api_key'] = api_key
+        if agent is not None:
+            init_panrc['agent'] = agent
 
         try:
             panrc = pan.rc.PanRc(tag=self.tag,
@@ -177,6 +140,8 @@ class PanWFapi:
 
         if 'api_key' in panrc.panrc:
             self.api_key = panrc.panrc['api_key']
+        if 'agent' in panrc.panrc:
+            self.agent = panrc.panrc['agent']
         if 'hostname' in panrc.panrc:
             self.hostname = panrc.panrc['hostname']
         else:
@@ -190,9 +155,6 @@ class PanWFapi:
         else:
             self.uri = 'https://%s' % self.hostname
 
-        if _legacy_urllib:
-            self._log(DEBUG2, 'using legacy urllib')
-
     def __str__(self):
         x = self.__dict__.copy()
         for k in x:
@@ -228,6 +190,9 @@ class PanWFapi:
               content_type == 'text/xml'):
             return self.__set_xml_response(message_body)
 
+        elif content_type == 'application/json':
+            return self.__set_json_response(message_body)
+
         elif content_type == 'text/html':
             return self.__set_html_response(message_body)
 
@@ -281,6 +246,18 @@ class PanWFapi:
 
         return True
 
+    def __set_json_response(self, message_body):
+        self._log(DEBUG2, '__set_json_response: %s', repr(message_body))
+        self.response_type = 'json'
+
+        _message_body = message_body.decode()
+        if len(_message_body) == 0:
+            return True
+
+        self.response_body = _message_body
+
+        return True
+
     def __set_html_response(self, message_body):
         self._log(DEBUG2, '__set_html_response: %s', repr(message_body))
         self.response_type = 'html'
@@ -320,23 +297,12 @@ class PanWFapi:
         self._log(DEBUG3, 'xml_root.decode(): %s', type(s.decode(_encoding)))
         return s.decode(_encoding)
 
-# XXX Unicode notes
-# 2.7
-# decode() str (bytes) -> unicode
-# encode() unicode -> str (bytes)
-# encode() of str will call decode()
-# 3.x
-# decode() bytes -> str (unicode)
-# encode() str (unicode) -> bytes
-# cannot encode() bytes
-# cannot decode() str
-
     def __api_request(self, request_uri, body, headers={}):
         url = self.uri
         url += request_uri
 
-        # body must by type 'bytes' for 3.x
-        if _isunicode(body):
+        # body must be type 'bytes'
+        if isinstance(body, str):
             body = body.encode()
 
         request = Request(url, body, headers)
@@ -350,7 +316,7 @@ class PanWFapi:
 
         kwargs = {
             'url': request,
-            }
+        }
 
         if self.ssl_context is not None:
             kwargs['context'] = self.ssl_context
@@ -360,7 +326,7 @@ class PanWFapi:
 
         try:
             response = self._urlopen(**kwargs)
-        except self._certificateerror as e:
+        except ssl.CertificateError as e:
             self._msg = 'ssl.CertificateError: %s' % e
             return False
         except (URLError, IOError) as e:
@@ -369,12 +335,7 @@ class PanWFapi:
             return False
 
         self.http_code = response.getcode()
-        if hasattr(response, 'reason'):
-            # 3.2
-            self.http_reason = response.reason
-        elif hasattr(response, 'msg'):
-            # 2.7
-            self.http_reason = response.msg
+        self.http_reason = response.reason
 
         if self.http_reason == '':
             if self.http_code in _wildfire_responses:
@@ -426,17 +387,22 @@ class PanWFapi:
 
     def report(self,
                hash=None,
-               format=None):
+               format=None,
+               url=None):
         self.__clear_response()
 
         request_uri = '/publicapi/get/report'
 
         query = {}
         query['apikey'] = self.api_key
+        if self.agent is not None:
+            query['agent'] = self.agent
         if hash is not None:
             query['hash'] = hash
         if format is not None:
             query['format'] = format
+        if url is not None:
+            query['url'] = url
 
         response = self.__api_request(request_uri=request_uri,
                                       body=urlencode(query))
@@ -447,15 +413,20 @@ class PanWFapi:
             raise PanWFapiError(self._msg)
 
     def verdict(self,
-                hash=None):
+                hash=None,
+                url=None):
         self.__clear_response()
 
         request_uri = '/publicapi/get/verdict'
 
         query = {}
         query['apikey'] = self.api_key
+        if self.agent is not None:
+            query['agent'] = self.agent
         if hash is not None:
             query['hash'] = hash
+        if url is not None:
+            query['url'] = url
 
         response = self.__api_request(request_uri=request_uri,
                                       body=urlencode(query))
@@ -473,6 +444,8 @@ class PanWFapi:
 
         form = _MultiPartFormData()
         form.add_field('apikey', self.api_key)
+        if self.agent is not None:
+            form.add_field('agent', self.agent)
         if hashes is not None:
             form.add_field('file', '\n'.join(hashes))
 
@@ -495,6 +468,8 @@ class PanWFapi:
 
         query = {}
         query['apikey'] = self.api_key
+        if self.agent is not None:
+            query['agent'] = self.agent
         if date is not None:
             query['date'] = date
 
@@ -514,6 +489,8 @@ class PanWFapi:
 
         query = {}
         query['apikey'] = self.api_key
+        if self.agent is not None:
+            query['agent'] = self.agent
         if hash is not None:
             query['hash'] = hash
 
@@ -534,6 +511,8 @@ class PanWFapi:
 
         query = {}
         query['apikey'] = self.api_key
+        if self.agent is not None:
+            query['agent'] = self.agent
         if hash is not None:
             query['hash'] = hash
         if platform is not None:
@@ -583,6 +562,8 @@ class PanWFapi:
 
         form = _MultiPartFormData()
         form.add_field('apikey', self.api_key)
+        if self.agent is not None:
+            form.add_field('agent', self.agent)
 
         if file is not None:
             buf = self._read_file(file)
@@ -629,6 +610,8 @@ class PanWFapi:
 
         form = _MultiPartFormData()
         form.add_field('apikey', self.api_key)
+        if self.agent is not None:
+            form.add_field('agent', self.agent)
         if hash is not None:
             form.add_field('hash', hash)
         if verdict is not None:
@@ -649,6 +632,30 @@ class PanWFapi:
         if not self.__set_response(response):
             raise PanWFapiError(self._msg)
 
+    def web_artifacts(self, *,
+                      url=None,
+                      types=None):
+        self.__clear_response()
+
+        request_uri = '/publicapi/get/webartifacts'
+
+        query = {}
+        query['apikey'] = self.api_key
+        if self.agent is not None:
+            query['agent'] = self.agent
+        if url is not None:
+            query['url'] = url
+        if types is not None:
+            query['types'] = types
+
+        response = self.__api_request(request_uri=request_uri,
+                                      body=urlencode(query))
+        if not response:
+            raise PanWFapiError(self._msg)
+
+        if not self.__set_response(response):
+            raise PanWFapiError(self._msg)
+
     # allow non-2XX error codes
     # see http://bugs.python.org/issue18543 for why we can't just
     # install a new HTTPErrorProcessor()
@@ -673,15 +680,11 @@ class PanWFapi:
         return opener.open(url, data, timeout)
 
     def _certifi_ssl_context(self):
-        if (sys.version_info.major == 2 and sys.hexversion >= 0x02070900 or
-           sys.version_info.major == 3 and sys.hexversion >= 0x03040300):
-            where = certifi.where()
-            self._log(DEBUG1, 'certifi %s: %s', certifi.__version__, where)
-            return ssl.create_default_context(
-                purpose=ssl.Purpose.SERVER_AUTH,
-                cafile=where)
-        else:
-            return None
+        where = certifi.where()
+        self._log(DEBUG1, 'certifi %s: %s', certifi.__version__, where)
+        return ssl.create_default_context(
+            purpose=ssl.Purpose.SERVER_AUTH,
+            cafile=where)
 
 
 #
@@ -700,7 +703,7 @@ def cloud_ssl_context():
     #   $ openssl x509 -in wfapi.py -text
     # to view text form.
 
-    gd_class2_root_crt = b'''
+    gd_class2_root_crt = '''
 -----BEGIN CERTIFICATE-----
 MIIEADCCAuigAwIBAgIBADANBgkqhkiG9w0BAQUFADBjMQswCQYDVQQGEwJVUzEh
 MB8GA1UEChMYVGhlIEdvIERhZGR5IEdyb3VwLCBJbmMuMTEwLwYDVQQLEyhHbyBE
@@ -727,15 +730,9 @@ ReYNnyicsbkqWletNw+vHX/bvZ8=
 -----END CERTIFICATE-----
 '''
 
-    if (sys.version_info.major == 2 and sys.hexversion >= 0x02070900 or
-            sys.version_info.major == 3 and sys.hexversion >= 0x03040300):
-        # XXX python >= 2.7.9 needs catada as Unicode, or we get:
-        # 'ssl.SSLError: nested asn1 error'
-        return ssl.create_default_context(
-            purpose=ssl.Purpose.SERVER_AUTH,
-            cadata=gd_class2_root_crt.decode())
-    else:
-        return None
+    return ssl.create_default_context(
+        purpose=ssl.Purpose.SERVER_AUTH,
+        cadata=gd_class2_root_crt)
 
 
 # Minimal RFC 2388 implementation
@@ -801,7 +798,7 @@ class _MultiPartFormData:
         headers = {
             'Content-Type':
                 'multipart/form-data; boundary=' + boundary,
-            }
+        }
 
         return headers
 
@@ -845,10 +842,9 @@ class _FormDataPart:
             s = '%s="%s"' % (name, value)
             self._log(DEBUG1, '_FormDataPart._encode_field: %s %s',
                       type(s), s)
-            if _isunicode(s):
-                s = s.encode('utf-8')
-                self._log(DEBUG1, '_FormDataPart._encode_field: %s %s',
-                          type(s), s)
+            s = s.encode('utf-8')
+            self._log(DEBUG1, '_FormDataPart._encode_field: %s %s',
+                      type(s), s)
             return s
 
         if not [ch for ch in '\r\n\\' if ch in value]:
@@ -863,7 +859,7 @@ class _FormDataPart:
         return ('%s*=%s' % (name, value)).encode('ascii')
 
     def add_body(self, body):
-        if _isunicode(body):
+        if isinstance(body, str):
             body = body.encode('latin-1')
         self.body = body
         self._log(DEBUG1, '_FormDataPart.add_body: %s %d',
